(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
    typeof define === 'function' && define.amd ? define(['exports'], factory) :
    (factory((global.ngrxUndo = global.ngrxUndo || {})));
}(this, (function (exports) { 'use strict';

var UNDO_ACTION = 'ngrx-undo/UNDO_ACTION';
function undo(action) {
    return {
        type: UNDO_ACTION,
        payload: action
    };
}

var STORE_INIT_ACTION = '@ngrx/store/init';
var STORE_UPDATE_ACTION = '@ngrx/store/update-reducers';
var executedActions = [];
var initialState;
var bufferSize = 100;
function configureBufferSize(size) {
    bufferSize = size;
}
function handleUndo(rootReducer) {
    return function (state, action) {
        if (action.type === UNDO_ACTION) {
            // if the action is UNDO_ACTION,
            // then call all the actions again on the rootReducer,
            // except the one we want to rollback
            var newState_1 = initialState;
            executedActions = executedActions.filter(function (eAct) { return eAct !== action.payload; });
            // update the state for every action untill we get the
            // exact same state as before, but without the action we want to rollback
            executedActions.forEach(function (executedAction) {
                return newState_1 = rootReducer(newState_1, executedAction);
            });
            return newState_1;
        }
        if (!(action.type === STORE_INIT_ACTION || action.type === STORE_UPDATE_ACTION)) {
            // push every action that isn't an UNDO_ACTION, STORE_INIT_ACTION, or STORE_UPDATE_ACTION to the executedActions property
            executedActions.push(action);
        }
        var updatedState = rootReducer(state, action);
        if (executedActions.length === bufferSize + 1) {
            var firstAction = executedActions[0];
            // calculate the state x (buffersize) actions ago
            initialState = rootReducer(initialState, firstAction);
            // keep the correct actions
            executedActions = executedActions.slice(1, bufferSize + 1);
        }
        return updatedState;
    };
}

exports.configureBufferSize = configureBufferSize;
exports.handleUndo = handleUndo;
exports.UNDO_ACTION = UNDO_ACTION;
exports.undo = undo;

Object.defineProperty(exports, '__esModule', { value: true });

})));